"use strict";

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/**
 * @class
 * @classdesc a base class that is to be extended by all the view classes
 * */
var AdharaView = function () {

    /**
     * @constructor
     * @param {AdharaView} parentViewInstance - parent view instance that is to be passed to render a view inside another.
     * */
    function AdharaView(parentViewInstance) {
        _classCallCheck(this, AdharaView);

        this._parentView = parentViewInstance;
        Adhara.instances[this.constructor.name] = this;
        this._data = null;
        this._error = null;
        this._event_listeners = {};
        this._registerEvents(["ViewRendered"]);
        this._registerEvents(this.events);
    }

    /**
     * @getter
     * @instance
     * @returns {Array<String>} return list of custom event names in PascalCase...
     * @description event listener registration functions will be created dynamically by prepending "on" th the event name
     * @example
     * say get events() returns this array: ["Render", "Format"]
     * listener functions these events can be then registered using `onRender(listener)` and `onFormat(listener)`
     * */


    _createClass(AdharaView, [{
        key: "_registerEvents",


        /**
         * @function
         * @protected
         * @param {Array<String>} event_names - list of events that are to be enabled on the current View instance.
         * @description registers the event names and creates event registration functions.
         * @example
         * _registerEvents(["Render", "Format"]);
         * //This will create 2 functions onRender and onFormat runtime to allow registration of events
         * */
        value: function _registerEvents(event_names) {
            var _this = this;

            var _iteratorNormalCompletion = true;
            var _didIteratorError = false;
            var _iteratorError = undefined;

            try {
                var _loop = function _loop() {
                    var event_name = _step.value;

                    _this._event_listeners[event_name] = [];
                    _this["on" + event_name] = function (handler) {
                        _this._event_listeners[event_name].push(handler);
                    };
                };

                for (var _iterator = event_names[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
                    _loop();
                }
            } catch (err) {
                _didIteratorError = true;
                _iteratorError = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion && _iterator.return) {
                        _iterator.return();
                    }
                } finally {
                    if (_didIteratorError) {
                        throw _iteratorError;
                    }
                }
            }
        }
    }, {
        key: "trigger",
        value: function trigger(event_name) {
            for (var _len = arguments.length, data = Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
                data[_key - 1] = arguments[_key];
            }

            var _iteratorNormalCompletion2 = true;
            var _didIteratorError2 = false;
            var _iteratorError2 = undefined;

            try {
                for (var _iterator2 = this._event_listeners[event_name][Symbol.iterator](), _step2; !(_iteratorNormalCompletion2 = (_step2 = _iterator2.next()).done); _iteratorNormalCompletion2 = true) {
                    var event_handler = _step2.value;

                    event_handler(data);
                }
            } catch (err) {
                _didIteratorError2 = true;
                _iteratorError2 = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion2 && _iterator2.return) {
                        _iterator2.return();
                    }
                } finally {
                    if (_didIteratorError2) {
                        throw _iteratorError2;
                    }
                }
            }
        }

        /**
         * @method
         * @getter
         * @returns {HandlebarTemplate} template of the view
         * */

    }, {
        key: "fetchData",


        /**
         * @function
         * @instance
         * @description hook to make API calls, data change event will be triggered on successful API call.
         * By default no API call will be made and dataChange method will be called right away.
         * */
        value: function fetchData() {
            var config = this.entityConfig;
            if (config) {
                if (dataInterface.getHTTPMethod(config.data_config.default_query_type) === "get") {
                    Controller.control(config.data_config.default_query_type, config, config.data);
                } else {
                    this.handleDataChange();
                }
            } else {
                this.handleDataChange();
            }
        }

        /**
         * @function
         * @instance
         * @param {DataBlob|Array<DataBlob>} [new_data=null] - Data in the form of DataBlob instance to be updated in view
         * */

    }, {
        key: "handleDataChange",
        value: function handleDataChange(new_data) {
            this.dataChange(new_data);
            this.render();
        }

        /**
         * @function
         * @instance
         * @param {*} error - Error to be updated in view
         * */

    }, {
        key: "handleDataError",
        value: function handleDataError(error) {
            this.dataError(error);
            this.render();
        }

        /**
         * @function
         * @instance
         * @param {DataBlob} new_data - Data in the form of DataBlob instance to be set in view
         * */

    }, {
        key: "dataChange",
        value: function dataChange(new_data) {
            this._data = new_data;
            this._error = null;
        }

        /**
         * @function
         * @instance
         * @param {*} error - Error to be set in view
         * */

    }, {
        key: "dataError",
        value: function dataError(error) {
            this._error = error;
        }

        /**
         * @method
         * @getter
         * @returns {*} View errors, that can be consumed by the template.
         * */

    }, {
        key: "_getHTML",
        value: function _getHTML(template) {
            return HandlebarUtils.execute(template || this.template, this);
        }
    }, {
        key: "_getParentContainer",
        value: function _getParentContainer() {
            var container = this.parentView.contentContainer;
            if (typeof container === "string") {
                return container;
            }
            return container[this.constructor.name] || container["*"];
        }
    }, {
        key: "getParentContainerElement",
        value: function getParentContainerElement() {
            return document.querySelector(this._getParentContainer());
        }
    }, {
        key: "render",
        value: function render() {
            var _this2 = this;

            var container = this.getParentContainerElement();
            if (!container) {
                return;
            }
            container.innerHTML = this._getHTML();
            setTimeout(function () {
                _this2._format(container);
                _this2.format(container);
            }, 0);
            this.renderSubViews();
            this.trigger("ViewRendered");
        }

        /**
         * @function
         * @getter
         * @returns {String} A CSS selector inside which child views are to be rendered.
         * */

    }, {
        key: "renderSubViews",
        value: function renderSubViews() {
            var _iteratorNormalCompletion3 = true;
            var _didIteratorError3 = false;
            var _iteratorError3 = undefined;

            try {
                for (var _iterator3 = this.subViews[Symbol.iterator](), _step3; !(_iteratorNormalCompletion3 = (_step3 = _iterator3.next()).done); _iteratorNormalCompletion3 = true) {
                    var sub_view = _step3.value;

                    Adhara.createView(Adhara.getView(sub_view, this));
                }
            } catch (err) {
                _didIteratorError3 = true;
                _iteratorError3 = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion3 && _iterator3.return) {
                        _iterator3.return();
                    }
                } finally {
                    if (_didIteratorError3) {
                        throw _iteratorError3;
                    }
                }
            }
        }
    }, {
        key: "_format",
        value: function _format(container) {
            var _this3 = this;

            var onClickElements = container.querySelectorAll("[data-onclick]");
            var _iteratorNormalCompletion4 = true;
            var _didIteratorError4 = false;
            var _iteratorError4 = undefined;

            try {
                var _loop2 = function _loop2() {
                    var onClickElement = _step4.value;

                    onClickElement.addEventListener("click", function (event) {
                        return _this3[onClickElement.dataset.onclick](event);
                    });
                };

                for (var _iterator4 = onClickElements[Symbol.iterator](), _step4; !(_iteratorNormalCompletion4 = (_step4 = _iterator4.next()).done); _iteratorNormalCompletion4 = true) {
                    _loop2();
                }
            } catch (err) {
                _didIteratorError4 = true;
                _iteratorError4 = err;
            } finally {
                try {
                    if (!_iteratorNormalCompletion4 && _iterator4.return) {
                        _iterator4.return();
                    }
                } finally {
                    if (_didIteratorError4) {
                        throw _iteratorError4;
                    }
                }
            }
        }
    }, {
        key: "format",
        value: function format(container) {
            //Control the DOM elements after rendering
        }
    }, {
        key: "refresh",
        value: function refresh() {
            Adhara.createView(this);
        }
    }, {
        key: "events",
        get: function get() {
            return [];
        }
    }, {
        key: "template",
        get: function get() {
            if (AdharaRouter.getCurrentPageName()) {
                return AdharaRouter.getCurrentPageName().replace(/_/g, "-");
            } else {
                var className = this.constructor.name.toLowerCase();
                if (className.endsWith("view")) {
                    return className.slice(0, className.length - 4);
                } else {
                    return className;
                }
            }
        }
    }, {
        key: "entityConfig",
        get: function get() {
            var entity_name = Adhara.view_context[this.constructor.name];
            return entity_name ? Adhara.app.getEntityConfig(entity_name) : null;
        }
    }, {
        key: "errors",
        get: function get() {
            return this._error;
        }

        /**
         * @method
         * @getter
         * @returns {*} View data, that can be consumed by the template.
         * */

    }, {
        key: "data",
        get: function get() {
            return this._data;
        }
    }, {
        key: "parentView",
        get: function get() {
            return this._parentView || Adhara.getView(Adhara.app.containerView);
        }
    }, {
        key: "contentContainer",
        get: function get() {
            throw new Error("implement this method");
        }

        /**
         * @typedef {Object} SubView - Sub view configuration
         * @property {String} container_selector - container's CSS selector
         * @property {class} view - view class reference of the sub view
         * */

        /**
         * @function
         * @instance
         * @returns Array<SubView>
         * */

    }, {
        key: "subViews",
        get: function get() {
            return [];
        }
    }]);

    return AdharaView;
}();
//# sourceMappingURL=View.js.map